
// Amplifier Driver ExampleDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Amplifier Example.h"
#include "Amplifier ExampleDlg.h"
#include "afxdialogex.h"
#include "string"
#include "../../../Amplifier.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CAmplifierDriverExampleDlg dialog



CAmplifierDriverExampleDlg::CAmplifierDriverExampleDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(IDD_AMPLIFIERDRIVEREXAMPLE_DIALOG, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CAmplifierDriverExampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_VISA_RESOURCE_NAME, edt_VisaResourceName);
	DDX_Control(pDX, IDC_AMPLIFIER_MODEL, lst_Amplifier_Model);
	DDX_Control(pDX, IDC_RF_GAIN, edt_RF_Gain);
	DDX_Control(pDX, IDC_POWER_ON, chk_PowerOn);
	DDX_Control(pDX, IDC_RF_ON, chk_RF_On);
	DDX_Control(pDX, IDC_ID, edt_ID);
	DDX_Control(pDX, IDC_FORWARD_POWER, edt_ForwardPower);
	DDX_Control(pDX, IDC_REVERSE_POWER, edt_ReversePower);
	DDX_Control(pDX, IDC_STATE, edt_State);
	DDX_Control(pDX, IDC_FAULT, edt_Fault);
}

BEGIN_MESSAGE_MAP(CAmplifierDriverExampleDlg, CDialogEx)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_RUN_DRIVER, &CAmplifierDriverExampleDlg::OnBnClickedRunDriver)
END_MESSAGE_MAP()


// CAmplifierDriverExampleDlg message handlers

BOOL CAmplifierDriverExampleDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// Set some defaults to show format
	edt_VisaResourceName.SetWindowText(TEXT("GPIB0::4::INSTR"));
	edt_RF_Gain.SetWindowTextW(TEXT("20"));
	chk_PowerOn.SetCheck(BST_CHECKED);
	chk_RF_On.SetCheck(BST_CHECKED);;

	// Call GetValidModels to get a list of supported Amplifier models
	std::string strModels[200];
	int count;
	CString Name;

	Amplifier Driver;

	Driver.GetValidModels(strModels, &count);

	lst_Amplifier_Model.AddString(_T("LCD Touch Panel"));
	for (int i = 0; i < count; i++)
	{
		Name = strModels[i].c_str();
		lst_Amplifier_Model.AddString(Name);
	}

	lst_Amplifier_Model.SetCurSel(0);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CAmplifierDriverExampleDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialogEx::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CAmplifierDriverExampleDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}



void CAmplifierDriverExampleDlg::OnBnClickedRunDriver()
{
	// Initialize the Driver and open the VISA sesson to the amplifier
	// if using RS-232, set the parameters for Serial control in the Initialize
	// function (look at AmplifierDriver.h for format).

	CString VisaResource;
	char VisaResourceName[100] = "";
	edt_VisaResourceName.GetWindowTextW(VisaResource);
	ConvertToChar(VisaResource, VisaResourceName, VisaResource.GetLength());
	
	CString ListString;
	char ModelName[100] = "";
	lst_Amplifier_Model.GetText(lst_Amplifier_Model.GetCurSel(), ListString);
	ConvertToChar(ListString, ModelName, 50);

	Amplifier Driver(ModelName);
	Driver.Initialize(VisaResourceName);
	Sleep(100);

	// Identity query
	CString strID(Driver.QueryID().c_str());
	edt_ID.SetWindowTextW(strID);

	// Set the Gain
	CString strGain;
	edt_RF_Gain.GetWindowTextW(strGain);
	unsigned int Gain = _ttoi(strGain);
	Driver.ConfigureRFGain(Gain);
	Sleep(500);

	// Turn the power on or off.  Note: some amplifiers require more time after
	// powering on before other commands can be sent
	int status;
	if (chk_PowerOn.GetCheck() == BST_CHECKED)
	{
		status = Driver.Power(true);
		if (status == 0)
			Sleep(2000);
	}
	else
		status = Driver.Power(false);

	//Turn the RF on or off
	// In this example the RF will only be enabled
	// if the power is turned on
	if (chk_PowerOn.GetCheck() == BST_CHECKED && chk_RF_On.GetCheck() == BST_CHECKED)
	{
		Driver.RF_Output(true);
	}
	else
	{
		Driver.RF_Output(false);
	}
	Sleep(100);

	// Read the status of the amplifier.
	// Refer to the manual for tables showing possible values
	// and their meaning for the fault and state commands
	unsigned int state = Driver.ReadState();
	Sleep(100);
	CString strState;
	strState.Format(TEXT("%x"), state);
	edt_State.SetWindowTextW(strState);

	unsigned int fault = Driver.ReadFault();
	Sleep(100);
	CString strFault;
	strFault.Format(TEXT("%x"), fault);
	edt_Fault.SetWindowTextW(strFault);

	// Read the forward and reverse power if no faults were reported
	double ForwardPower = 0.0;
	double ReversePower = 0.0;
	if (fault == 0)
	{
		ForwardPower = Driver.QueryForwardPower();
		ReversePower = Driver.QueryReversePower();
	}

	CString strForwardPower, strReversePower;
	strForwardPower.Format(TEXT("%f"), ForwardPower);
	edt_ForwardPower.SetWindowTextW(strForwardPower);

	strReversePower.Format(TEXT("%f"), ReversePower);
	edt_ReversePower.SetWindowTextW(strReversePower);

	// Close communication with the amplifier
	Driver.Close();

}

char* CAmplifierDriverExampleDlg::ConvertToChar(const CString &s, char* pAnsiString, int bufsize)
{
	wcstombs(pAnsiString, (LPCTSTR)s, bufsize);
	return pAnsiString;
}

