#pragma once

#include <visa.h>
#include <string>
#include <sstream>
#include <array>
#include <vector>

class SystemController
{
public:
	enum ParityType { NONE, ODD, EVEN, MARK, SPACE };
	enum FlowControlType { NO_FLOW_CONTROL, XON_XOFF, RTS_CTS, DTR_DSR };
	enum StopBitsType { ONE, ONE5, TWO };

	int Initialize(std::string VisaResourceName,
		bool reset = true,
		int BaudRate = 9600,
		FlowControlType FlowControl = NO_FLOW_CONTROL,
		ParityType Parity = NONE,
		int DataBits = 8,
		StopBitsType StopBits = ONE);
	int Reset();
	void SetSwitch(int sw, int position);
	int QuerySwitch(int sw);
	int Close();

protected:
	ViSession VISAresource;

	int SendCommand(std::string command, std::string &response, bool Query = false, unsigned int waitAfterSend = 20, unsigned int BytesToRead = 100);
};

class SC2000 : public SystemController
{
public:
	enum ConfigName { USER1, USER2, USER3, USER4, USER5, USER6, USER7, USER8, INT1, INT2, INT3, SAFE };

	std::string QueryID();
	std::array<bool,4> ReadState();
	int ReadFault();
	void LocalMode();

	void SetController(int controller);
	void SetRelay(int bank, int relay, int position);
	void SetOpenDrain(int openDrain, bool active);
	void SetTTL(int ttl, bool state);

	void SetSwitches(std::vector<std::vector<int>> switches, bool waitForComplete = true, int timeout = 10);
	void SetOpenDrains(std::vector<std::vector<int>> openDrains, bool waitForComplete = true, int timeout = 10);
	void SetTTLs(std::vector<std::vector<int>> ttls, bool waitForComplete = true, int timeout = 10);

	int QueryController();
	int QuerySwitchCapability(int sw);
	int QueryRelay(int bank, int relay);
	bool QueryOpenDrain(int openDrain);
	bool QueryTTL(int ttl);

	void SaveConfiguration(ConfigName name);
	void LoadConfiguration(ConfigName name, bool waitForComplete = true, int timeout = 10);

	void WaitForSwitching(int timeout);

private:
	const std::string ConfigNameString[12] = { "USER1", "USER2", "USER3", "USER4", "USER5", "USER6", "USER7", "USER8", "INT1", "INT2", "INT3", "SAFE" };
	bool ParseState(int state, int bit);
	int Clamp(int input, int min, int max);
};

class SC1000 : public SystemController
{
public:
	void SetSwitch(int sw, int position);
	bool ReadFault();
	void Power(bool state);
	void SetOpenCollector(int openCollector, bool active);
	void Set12V(bool enable);

	bool QueryOpenCollector(int openCollector);
	bool Query12V();
};
